
################################################################################
# 
#                    NEC CORPORATION PROPRIETARY INFORMATION
#        This software is supplied under the terms of a license agreement or
#        nondisclosure agreement with NEC Corporation and may not be copied
#        or disclosed except in accordance with the terms of that agreement.
#          Copyright (c) 2015 NEC Corporation. All Rights Reserved.
# 
################################################################################

package CommonModule;

use strict;
use warnings;
# HTTPアクセス用モジュールをインポート
use LWP::UserAgent;
use HTTP::Request::Common qw(GET);
use HTTP::Request::Common qw(POST);
# JSON解析用モジュールをインポート
use JSON;
# SHA1算出用モジュールをインポート
use Digest::SHA;
# ファイルパスの解析
use File::Basename;

use constant USERNAMELEN_MAX  => 16;
use constant PASSWORDLEN_MAX  => 20;
use constant CHALLENGELEN_MAX => 16;
use constant LOCALADDR => "\x7F\x00\x00\x01";
use constant LOGINRETRYINTERVAL => 15;
use constant GETTOKENRETRYINTERVAL => 5;

my $usage = "";
my %filelist =
(
	"basic"           => "basic.json" ,
	"useraccount"     => "useraccount.json" ,
	"network"         => "network.json" ,
	"alert"           => "alert.json" ,
	"operation"       => "operation.json" ,
	"eco"             => "eco.json" ,
	"bios"            => "bios.json" ,
	"battery"         => "battery.json"
);


sub new {
	my $class = shift;
	my $self = {@_} ;
	
	if ( !defined($self->{host}) )
	{
		print "'host' is required.\n";
		return;
	}
	if ( !defined($self->{username}) )
	{
		print "'username' is required.\n";
		return;
	}
	if ( !defined($self->{password}) )
	{
		print "'password' is required.\n";
		return;
	}
	if ( !defined($self->{ssl}) )
	{
		$self->{ssl}=0;
	}
	if ( !defined($self->{skipcertcheck}) )
	{
		$self->{skipcertcheck}=0;
	}
	if ( !defined($self->{certfile}) )
	{
		$self->{certfile}='';
	}
	
	if ($self->{ssl})
	{
		# HTTPS
		$self->{url} = "https://" . $self->{host};
	}
	else
	{
		# HTTP
		$self->{url} = "http://" . $self->{host};
	}
	
	bless $self, $class;
	
	return $self;
}


sub getConfig {
	my $self = shift;
	my $type;
	my $filename;
	my $json;
	my $res;
	my $checkcert = !($self->{skipcertcheck});
	
	($type, $filename) = @_;
	
	if ( &checkParam( $self->{host} , $self->{username} , $self->{password} , $type ) != 0 )
	{
		return -1;
	}
	
	( $res , $json ) = &getbmccfg($self->{username} , $self->{password}, $self->{url}, $filename , $type, $checkcert ,$self->{certfile});
	
	return ( $res , $json );
}


sub setConfig {
	my $self = shift;
	my $url;
	my $type;
	my $filename;
	my $res;
	my $checkcert = !($self->{skipcertcheck});
	
	($type , $filename) = @_;
	
	if ( &checkParam( $self->{host} , $self->{username} , $self->{password} , $type ) != 0 )
	{
		return -1;
	}
	
	if ( !defined($filename) )
	{
		print "'filename' is required.\n";
		return -1;
	}
	
	if (!-f $filename)
	{
		print "'$filename' is not found.\n";
		return -1;
	}

	$res = &setbmccfg($self->{username} , $self->{password}, $self->{url} , $filename , $type, $checkcert , $self->{certfile});
	
	return $res;
}


sub getServerSetting {
	my $self = shift;
	my $xml;
	my $res;
	my $checkcert = !($self->{skipcertcheck});
	my $filename = '';
	
	$filename = $_[0];

	( $res , $xml ) = &getOobHw( $self->{username} , $self->{password} , $self->{url} , $checkcert , $self->{certfile} );
	
	if (($res==0)&&(defined ($filename))&&($filename ne ''))
	{
		open (OUT, "> $filename");
		print OUT $xml;
		close (OUT);
		print "'$filename' is successfully created.\n";
	}
	
	return ( $res , $xml );
}


sub setServerSetting {
	my $self = shift;
	my $xml;
	my $res;
	my $checkcert = !($self->{skipcertcheck});
	
	my $filename = $_[0];

	if ( !defined($filename) )
	{
		print "'filename' is required.\n";
		return ( -1 , '' );
	}
	
	if ( !-f $filename )
	{
		print "'$filename' is not found.\n";
		return ( -1 , '' );
	}

	( $res , $xml ) = &setOobHw( $self->{username} , $self->{password} , $self->{url} , $checkcert , $filename , $self->{certfile} );

	return ( $res , $xml );
}


sub getOobHw {
	my $username;
	my $password;
	my $url;
	my $checkcert;
	my $path;
	my $cookie;
	my $ret;
	my $content;
	my $csrftoken;
	my @csrftokenhdr;
	my $certpath;

	( $username , $password , $url , $checkcert , $certpath ) = @_;

	# BMCに対してログイン処理を行ない、クッキーを入手します
	($ret , $cookie) = &login($username, $password, $url, $checkcert, $certpath);
	if (0 != $ret)
	{
		# 1回だけ自動リトライ
		sleep LOGINRETRYINTERVAL;
		($ret , $cookie) = &login($username, $password, $url, $checkcert, $certpath);
		if (0 != $ret)
		{
			print "Failed to login.\n";
			if (($ret == -2)||($ret == -6)){
				print "Invalid username or password.\n";
				if ( $url=~m|^http://| ){
					print "SSL is required for LDAP / Active Directory user.\n";
				}
				$ret = -2;
			}
			if ($ret == -3){
				print "Session slot is full.\n";
			}
			if ($ret == -1){
				$ret=-3;
			}
			return ($ret,'');
		}
	}

	# CSRFトークンを入手します
	($ret , $csrftoken) = &getcsrftoken($cookie, $url, $checkcert, $certpath);
	if ( (0 != $ret) && ( -2 != $ret ) )
	{
		# 1回だけ自動リトライ
		&logout($cookie, $csrftoken, $url);
		sleep GETTOKENRETRYINTERVAL;
		($ret , $cookie) = &login($username, $password, $url, $checkcert, $certpath);
		if (0 != $ret)
		{
			print "Failed to login.\n";
			if (($ret == -2)||($ret == -6)){
				print "Invalid username or password.\n";
				if ( $url=~m|^http://| ){
					print "SSL is required for LDAP / Active Directory user.\n";
				}
				$ret = -2;
			}
			if ($ret == -3){
				print "Session slot is full.\n";
			}
			if ($ret == -1){
				$ret=-3;
			}
			return ($ret,'');
		}
		($ret , $csrftoken) = &getcsrftoken($cookie, $url, $checkcert, $certpath);
		if ( (0 != $ret) && ( -2 != $ret ) )
		{
			print "Failed to get CSRF token.\n";
			&logout($cookie, $csrftoken, $url);
			return (-1,'');
		}
	}
	@csrftokenhdr = ("CSRFTOKEN" => "$csrftoken");

	$path = "$url/rpc/server-config";

	# BMCからOOB HW情報を入手する本処理を行ないます
	{
		my @cookiehdr = ( "Cookie" => "SessionCookie=$cookie" );
		my $ua = LWP::UserAgent->new;
		my $statname = "HAPI_STATUS";
		my $req;
		my $res;

		# 証明書の設定を行います
		if ( $url=~m|^https://| ){
			if ( $checkcert ){
				if ( $certpath ne '' ){
					$ua->ssl_opts(SSL_ca_file => $certpath);
				}
			} else {
				$ua->ssl_opts(verify_hostname => 0, SSL_verify_mode => 'SSL_VERIFY_NONE');
			}
		}
		
		# ログイン処理で得たクッキーをヘッダに付加し、GET要求を発行します
		$req = GET( $path , @cookiehdr , @csrftokenhdr );

		$res = $ua->request( $req );
		if ( !$res->is_success )
		{
			print $res->status_line,"\n";
			$ret=-3;
			if ( $res->code == 404 ){
				print "Unsupported function.\n";
				$ret=-2;
			} elsif ( $res->code >= 400 )
			{
				$ret = &OutputHttpErrorStatusMessage( $res->code );
			}
			&logout( $cookie , $csrftoken , $url  );
			return ( $ret , '' );
		}
		$content = $res->content;
	}

	# BMCに対してログアウト処理を行ないます
	$ret = &logout( $cookie , $csrftoken , $url  );
	if ( 0 != $ret )
	{
		return ( $ret , '' );
	}

	return ( 0 , $content );
}


sub setOobHw {
	my $username;
	my $password;
	my $url;
	my $checkcert;
	my $filename;
	my $path;
	my $cookie;
	my $ret;
	my $content;
	my $csrftoken;
	my @csrftokenhdr;
	my $certpath;

	( $username , $password , $url , $checkcert, $filename, $certpath ) = @_;

	# BMCに対してログイン処理を行ない、クッキーを入手します
	($ret , $cookie) = &login($username, $password, $url, $checkcert, $certpath);
	if (0 != $ret)
	{
		# 1回だけ自動リトライ
		sleep LOGINRETRYINTERVAL;
		($ret , $cookie) = &login($username, $password, $url, $checkcert, $certpath);
		if (0 != $ret)
		{
			print "Failed to login.\n";
			if (($ret == -2)||($ret == -6)){
				print "Invalid username or password.\n";
				if ( $url=~m|^http://| ){
					print "SSL is required for LDAP / Active Directory user.\n";
				}
				$ret = -2;
			}
			if ($ret == -3){
				print "Session slot is full.\n";
			}
			if ($ret == -1){
				$ret=-3;
			}
			return ($ret,'');
		}
	}

	# CSRFトークンを入手します
	($ret , $csrftoken) = &getcsrftoken($cookie, $url, $checkcert, $certpath);
	if ( (0 != $ret) && ( -2 != $ret ) )
	{
		# 1回だけ自動リトライ
		&logout($cookie, $csrftoken, $url);
		sleep GETTOKENRETRYINTERVAL;
		($ret , $cookie) = &login($username, $password, $url, $checkcert, $certpath);
		if (0 != $ret)
		{
			print "Failed to login.\n";
			if (($ret == -2)||($ret == -6)){
				print "Invalid username or password.\n";
				if ( $url=~m|^http://| ){
					print "SSL is required for LDAP / Active Directory user.\n";
				}
				$ret = -2;
			}
			if ($ret == -3){
				print "Session slot is full.\n";
			}
			if ($ret == -1){
				$ret=-3;
			}
			return ($ret,'');
		}
		($ret , $csrftoken) = &getcsrftoken($cookie, $url, $checkcert, $certpath);
		if ( (0 != $ret) && ( -2 != $ret ) )
		{
			print "Failed to get CSRF token.\n";
			&logout($cookie, $csrftoken, $url);
			return (-1,'');
		}
	}
	@csrftokenhdr = ("CSRFTOKEN" => "$csrftoken");

	$path = "$url/rpc/server-config";

	# BMCからバックアップデータを入手する本処理を行ないます
	{
		my @cookiehdr = ( "Cookie" => "SessionCookie=$cookie" );
		my $ua = LWP::UserAgent->new;
		my $statname = "HAPI_STATUS";
		my $req;
		my $res;

		my $upname = '/mnt/oobxml.xml'.'?CSRFTOKEN='.$csrftoken;

		# 証明書の設定を行います
		if ( $url=~m|^https://| ){
			if ( $checkcert ){
				if ( $certpath ne '' ){
					$ua->ssl_opts(SSL_ca_file => $certpath);
				}
			} else {
				$ua->ssl_opts(verify_hostname => 0, SSL_verify_mode => 'SSL_VERIFY_NONE');
			}
		}

		# ログイン処理で得たクッキーをヘッダに付加し、POSTを発行します
		$req = POST( $path , @cookiehdr , @csrftokenhdr , Content_Type => 'multipart/form-data' ,
											Content => [ $upname => [$filename] ] );

		$res = $ua->request( $req );
		if ( !$res->is_success )
		{
			print $res->status_line,"\n";
			$ret=-3;
			if ( $res->code == 404 )
			{
				print "Unsupported function.\n";
				$ret=-2;
			} elsif ( $res->code >= 400 )
			{
				$ret = &OutputHttpErrorStatusMessage( $res->code );
			}
			&logout( $cookie , $csrftoken , $url );
			return ( $ret , '' );
		} else {
			print "'$filename' is successfully registered.\n";
		}
		$content = $res->content;
	}

	# BMCに対してログアウト処理を行ないます
	$ret = &logout( $cookie , $csrftoken , $url );
	if ( 0 != $ret )
	{
		return ( $ret , '' );
	}

	return ( 0 , $content );
}


sub checkParam {
	my $invalid = 0;
	my $username;
	my $password;
	my $host;
	my $type;
	
	($host , $username, $password, $type) = @_;
	
	if ( $host eq '' )
	{
		print "'host' is required.\n";
		$invalid = 1;
	}
	
	if ( $username eq '' )
	{
		print "'username' is required.\n";
		$invalid = 1;
	}
 	
	if ( $password eq '' )
	{
		print "'password' is required.\n";
		$invalid = 1;
	}
	
	if (length("$username") > USERNAMELEN_MAX)
	{
		print "'$username' is too long.\n";
		$invalid = 1;
	}
	if (length($password) > PASSWORDLEN_MAX)
	{
		print "'$password' is too long.\n";
		$invalid = 1;
	}
	
	if ( !(defined($filelist{$type})))
	{
		print "'$type' is invalid type.\n";
		$invalid = 1;
	}
	
	if ($invalid)
	{
		print $usage;
		return -1;
	}
	
	return 0;
}


# バックアップデータを入手
sub getbmccfg {
	my $username;
	my $password;
	my $url;
	my $type;
	my $checkcert;
	my $path;
	my $cookie;
	my $ret;
	my $filename;
	my $content;
	my $csrftoken;
	my @csrftokenhdr;
	my $certpath;
		
	($username, $password, $url, $filename, $type, $checkcert, $certpath) = @_;
	
	# 引数の正当性を確認します
	if (defined($filelist{$type}))
	{
		$path = $filelist{$type};
	}
	else
	{
		print "'$type' is invalid type.\n";
		return (-1,'');
	}
	
	$path = "$url/goform/$path";
	
	# BMCに対してログイン処理を行ない、クッキーを入手します
	($ret , $cookie) = &login($username, $password, $url, $checkcert, $certpath);
	if (0 != $ret)
	{
		# 1回だけ自動リトライ
		sleep LOGINRETRYINTERVAL;
		($ret , $cookie) = &login($username, $password, $url, $checkcert, $certpath);
		if (0 != $ret)
		{
			print "Failed to login.\n";
			if (($ret == -2)||($ret == -6)){
				print "Invalid username or password.\n";
				if ( $url=~m|^http://| ){
					print "SSL is required for LDAP / Active Directory user.\n";
				}
				$ret = -2;
			}
			if ($ret == -3){
				print "Session slot is full.\n";
			}
			if ($ret == -1){
				$ret=-3;
			}
			return ($ret,'');
		}
	}
	
	# CSRFトークンを入手します
	($ret , $csrftoken) = &getcsrftoken($cookie, $url, $checkcert, $certpath);
	if ( (0 != $ret) && ( -2 != $ret ) )
	{
		# 1回だけ自動リトライ
		&logout($cookie, $csrftoken, $url);
		sleep GETTOKENRETRYINTERVAL;
		($ret , $cookie) = &login($username, $password, $url, $checkcert);
		if (0 != $ret)
		{
			print "Failed to login.\n";
			if (($ret == -2)||($ret == -6)){
				print "Invalid username or password.\n";
				if ( $url=~m|^http://| ){
					print "SSL is required for LDAP / Active Directory user.\n";
				}
				$ret = -2;
			}
			if ($ret == -3){
				print "Session slot is full.\n";
			}
			if ($ret == -1){
				$ret=-3;
			}
			return ($ret,'');
		}
		($ret , $csrftoken) = &getcsrftoken($cookie, $url);
		if ( (0 != $ret) && ( -2 != $ret ) )
		{
			print "Failed to get CSRF token.\n";
			&logout($cookie, $csrftoken, $url);
			return (-1,'');
		}
	}
	@csrftokenhdr = ("CSRFTOKEN" => "$csrftoken");
	
	# BMCからバックアップデータを入手する本処理を行ないます
	{
		my @cookiehdr = ("Cookie" => "SessionCookie=$cookie");
		my $ua = LWP::UserAgent->new;
		my $index;
		my $statname = "HAPI_STATUS";
		my $req;
		my $res;
		my $result;
		
		# 証明書の設定を行います
		if ( $url=~m|^https://| ){
			if ( $checkcert ){
				if ( $certpath ne '' ){
					$ua->ssl_opts(SSL_ca_file => $certpath);
				}
			} else {
				$ua->ssl_opts(verify_hostname => 0, SSL_verify_mode => 'SSL_VERIFY_NONE');
			}
		}
		
		# ログイン処理で得たクッキーをヘッダに付加し、GET要求を発行します
		$req = GET($path, @cookiehdr, @csrftokenhdr);
		$res = $ua->request($req);
		if (!$res->is_success)
		{
			print $res->status_line,"\n";
			$ret=-3;
			if ( $res->code >= 400 )
			{
				$ret = &OutputHttpErrorStatusMessage( $res->code );
			}
			&logout($cookie, $csrftoken, $url, $checkcert, $certpath);
			return ( $ret , '' );
		}
		else
		{
			# 入手したバックアップデータをファイルに保存します
			$content = $res->content;
			if ( defined ($filename) && ( $filename ne '' ) )
			{
				open (OUT, "> $filename");
				print OUT $content;
				close (OUT);
				print "'$filename' is successfully created.\n";
			}
		}
	}

	# BMCに対してログアウト処理を行ないます
	$ret = &logout($cookie, $csrftoken, $url, $checkcert, $certpath);
	if (0 != $ret)
	{
		return ( $ret , '' );
	}

	return ( 0 , $content );
}


# バックアップデータを設定
sub setbmccfg
{
	my $username;
	my $password;
	my $url;
	my $filename;
	my $type;
	my $checkcert;
	my $path;
	my $cookie;
	my $name;
	my $ret;
	my $csrftoken;
	my @csrftokenhdr;
	my $certpath;
	
	($username, $password, $url, $filename, $type, $checkcert, $certpath) = @_;

	# 引数の正当性を確認します
	if (defined($filelist{$type}))
	{
		$name = '/tmp/' . $filelist{$type};
	}
	else
	{
		print "'$type' is invalid type.\n";
		return (-1);
	}

	# BMCに対してログイン処理を行ない、クッキーを入手します
	($ret , $cookie) = &login($username, $password, $url, $checkcert, $certpath);
	if (0 != $ret)
	{
		# 1回だけ自動リトライ
		sleep LOGINRETRYINTERVAL;
		($ret , $cookie) = &login($username, $password, $url, $checkcert, $certpath);
		if (0 != $ret)
		{
			print "Failed to login.\n";
			if (($ret == -2)||($ret == -6)){
				print "Invalid username or password.\n";
				if ( $url=~m|^http://| ){
					print "SSL is required for LDAP / Active Directory user.\n";
				}
				$ret = -2;
			}
			if ($ret == -3){
				print "Session slot is full.\n";
			}
			if ($ret == -1){
				$ret=-3;
			}
			return $ret;
		}
	}
	
	# CSRFトークンを入手します
	($ret , $csrftoken) = &getcsrftoken($cookie, $url, $checkcert, $certpath);
	if ( (0 != $ret) && ( -2 != $ret ) )
	{
		# 1回だけ自動リトライ
		&logout($cookie, $csrftoken, $url);
		sleep GETTOKENRETRYINTERVAL;
		($ret , $cookie) = &login($username, $password, $url, $checkcert);
		if (0 != $ret)
		{
			print "Failed to login.\n";
			if (($ret == -2)||($ret == -6)){
				print "Invalid username or password.\n";
				if ( $url=~m|^http://| ){
					print "SSL is required for LDAP / Active Directory user.\n";
				}
				$ret = -2;
			}
			if ($ret == -3){
				print "Session slot is full.\n";
			}
			if ($ret == -1){
				$ret=-3;
			}
			return ($ret,'');
		}
		($ret , $csrftoken) = &getcsrftoken($cookie, $url);
		if ( (0 != $ret) && ( -2 != $ret ) )
		{
			print "Failed to get CSRF token.\n";
			&logout($cookie, $csrftoken, $url);
			return (-1,'');
		}
	}
	@csrftokenhdr = ("CSRFTOKEN" => "$csrftoken");
	
	# BMCにバックアップデータを設定する本処理を行ないます
	{
		my @cookiehdr = ("Cookie" => "SessionCookie=$cookie");
		my @data;
		my $ua = LWP::UserAgent->new;
		my $index;
		my $content;
		my $strname = "WEBVAR_STRUCTNAME_RESTOREJSON";
		my $statname = "HAPI_STATUS";
		my $req;
		my $res;
		my $result;
		my $size;
		my $value;
		
		# 証明書の設定を行います
		if ( $url=~m|^https://| ){
			if ( $checkcert ){
				if ( $certpath ne '' ){
					$ua->ssl_opts(SSL_ca_file => $certpath);
				}
			} else {
				$ua->ssl_opts(verify_hostname => 0, SSL_verify_mode => 'SSL_VERIFY_NONE');
			}
		}
		
		# アップロードするファイルのサイズを入手
		$size = -s $filename;
		$name = sprintf("%s?%d", $name, $size);

		# ログイン処理で得たクッキーをヘッダに付加し、POST要求にてファイルをアップロードします
		@data = ($name => [$filename]); 
		$path = "$url/rpc/restorejson.asp";
		$req = POST($path, @cookiehdr, @csrftokenhdr,
							 Content_Type => "form-data", Content => \@data);
		$res = $ua->request($req);
		if (!$res->is_success)
		{
			print $res->status_line,"\n";
			$ret=-3;
			if ( $res->code >= 400 )
			{
				$ret = &OutputHttpErrorStatusMessage( $res->code );
			}
			&logout($cookie, $csrftoken, $url, $checkcert, $certpath);
			return $ret;
		}
		else
		{
			$content = $res->content;

			# BMCから入手した応答を整形します
			($ret, $content) = preparse($content, $strname, $statname);
			if (0 != $ret)
			{
				&logout($cookie, $csrftoken, $url, $checkcert, $certpath);
				return $ret;
			}

			# 応答をJSON形式として解析します
			$result = JSON->new->decode($content);
			if ($result->{Error})
			{
				print $result->{Error}{Message}, "\n";
				&logout($cookie, $csrftoken, $url, $checkcert, $certpath);
				return -3;
			}
			else
			{
				if (defined($result->{$statname}))
				{
					$value = $result->{$statname};
					if (0 != $value)
					{
						print "$statname error, code $value.\n";
						print "Failed to set JSON. Please retry a few minutes later.\n";
						&logout($cookie, $csrftoken, $url, $checkcert, $certpath);
						return -3;
					}
				}
				else
				{
					print "'$statname' is not found.\n";
					print "Invalid response.\n";
					&logout($cookie, $csrftoken, $url, $checkcert, $certpath);
					return -3;
				}
				print "'$filename' is successfully registered.\n";
			}
		}
	}

	# BMCに対してログアウト処理を行ないます
	$ret = &logout($cookie, $csrftoken, $url, $checkcert, $certpath);
	if (0 != $ret)
	{
		return $ret;
	}

	return 0;
}


# ログイン処理
sub login
{
	my $username;
	my $password;
	my $url;
	my $checkcert;
	my $path;
	my $cookie = "";
	my $ua = LWP::UserAgent->new;
	my $index;
	my $content;
	my $statname = "HAPI_STATUS";
	my $key;
	my $array;
	my $value;
	my $challenge = "";
	my $digest;
	my $ret;
	my $certpath;

	($username, $password, $url, $checkcert, $certpath) = @_;
	
	# 証明書の設定を行います
	if ( $url=~m|^https://| ){
		if ( $checkcert ){
			if ( $certpath ne '' ){
				$ua->ssl_opts(SSL_ca_file => $certpath);
			}
		} else {
			$ua->ssl_opts(verify_hostname => 0, SSL_verify_mode => 'SSL_VERIFY_NONE');
		}
	}
	
	# BMCからChallengeCodeを入手します
	{
		my $req;
		my $res;
		my $strname = "WEBVAR_STRUCTNAME_GETCHALLENGE";
		my $resname = "CHALLENGECODE";
		my $result;

		# GET要求を発行します
		$path = "$url/rpc/WEBSES/getchallenge.asp";
		$req = GET($path);
		$res = $ua->request($req);
		if (!$res->is_success)
		{
			print $path, " : ", $res->status_line, "\n";
			return (-1, $cookie);
		}
		else
		{
			if ($checkcert)
			{
				# 証明書が正当かどうか確認します
				if (defined $res->header("Client-SSL-Warning"))
				{
					print $res->header("Client-SSL-Warning"), "\n";
					print "Certificate verify failed.\n";
					return (-1, $cookie);
				}
			}

			$content = $res->content;

			# BMCから入手した応答を整形します
			($ret, $content) = preparse($content, $strname, $statname);
			if (0 != $ret)
			{
				return ($ret, $cookie);
			}

			# 応答をJSON形式として解析します
			$result = JSON->new->decode($content);
			if ($result->{Error})
			{
				print $path, " : ", $result->{Error}{Message}, "\n";
				return (-1, $cookie);
			}
			else
			{
				if (defined($result->{$statname}))
				{
					$value = $result->{$statname};
					if (0 != $value)
					{
						print $path, " : ", "$statname error, code $value.\n";
						print "Invalid response.\n";
						return (-1, $cookie);
					}
				}
				else
				{
					print $path, " : ", "'$statname' is not found.\n";
					print "Invalid response.\n";
					return (-1, $cookie);
				}
				if (defined($result->{$strname}))
				{
					$array = $result->{$strname};
					if (defined($array->[0]->{$resname}))
					{
						$value = $array->[0]->{$resname};
					}
					else
					{
						print $path, " : ", "'$resname' is not found.\n";
						return (-1, $cookie);
					}
				}
				else
				{
					print $path, " : ", "'$strname' is not found.\n";
					return (-1, $cookie);
				}
				# 入手したChallengeCodeを整形します
				for ($index=0; $index<CHALLENGELEN_MAX; $index++)
				{
					$challenge = sprintf("%s%c",$challenge,hex(substr($value, $index*2, 2)));
				}
			}
		}
	}
	# ユーザ名,パスワード,IPアドレス,ChallengeCodeから認証用のハッシュを生成します
	{
		my $ctx;
		my $addr = LOCALADDR;

		$ctx = Digest::SHA->new('SHA-1');
		$ctx->add($username);
		for ($index=length($username); $index<USERNAMELEN_MAX; $index++)
		{
			$ctx->add("\x00");
		}
		$ctx->add($password);
		for ($index=length($password); $index<PASSWORDLEN_MAX; $index++)
		{
			$ctx->add("\x00");
		}
		$ctx->add($addr);
		for ($index=0; $index<8; $index++)
		{
			$ctx->add("\x00");
		}
		$ctx->add($challenge);
		$digest = uc($ctx->hexdigest);
	}
	# ログインの本処理を行ないます
	{
		my $addr = "";
		my $req;
		my $res;
		my $strname = "WEBVAR_STRUCTNAME_WEB_SESSION";
		my $resname = "SESSION_COOKIE";
		my %header = ();
		my %formdata;
		my $result;

		for ($index=0; $index<4; $index++)
		{
			$addr = sprintf("%s%2.2X",$addr,ord(substr(LOCALADDR, $index, 1)));
		}

		# 必要なパラメータを付加し、POST要求を発行します
		$path = "$url/rpc/WEBSES/create.asp";
		if ( $url=~m|^https://| ){
			%formdata = ("WEBVAR_USERNAME" => $username,
							"WEBVAR_PASSWORD" => $password,
							"WEBVAR_IPADDR" => $addr);
		} else {
			%formdata = ("WEBVAR_USERNAME" => $username,
							"WEBVAR_IPADDR" => $addr,
							"WEBVAR_HASH" => $digest);
		}
		$req = POST($path, [%formdata]);
		$res = $ua->request($req);
		if (!$res->is_success)
		{
			print $path, " : ", $res->status_line,"\n";
			return (-1, $cookie);
		}
		else
		{
			$content = $res->content;

			# BMCから入手した応答を整形します
			($ret, $content) = preparse($content, $strname, $statname);
			if (0 != $ret)
			{
				return ($ret, $cookie);
			}

			# 応答をJSON形式として解析します
			$result = JSON->new->decode($content);
			if ($result->{Error})
			{
				print $path, " : ", $result->{Error}{Message}, "\n";
				return (-1, $cookie);
			}
			else
			{
				if (defined($result->{$statname}))
				{
					$value = $result->{$statname};
					if (0 != $value)
					{
						print $path, " : ", "$statname error, code $value.\n";
						return ($value, $cookie);
					}
				}
				else
				{
					print $path, " : ", "'$statname' is not found.\n";
					print "Invalid response.\n";
					return (-1, $cookie);
				}
				if (defined($result->{$strname}))
				{
					$array = $result->{$strname};
					if (defined($array->[0]->{$resname}))
					{
						$value = $array->[0]->{$resname};
					}
					else
					{
						print $path, " : ", "'$resname' is not found.\n";
						print "Invalid response.\n";
						return (-1, $cookie);
					}
				}
				else
				{
					print $path, " : ", "'$strname' is not found.\n";
					return (-1, $cookie);
				}
				# 入手したクッキーを呼び元へ渡します
				$cookie = $value;
			}
		}
	}

	return (0, $cookie);
}

# ログアウト処理
sub logout
{
	my $cookie;
	my $url;
	my $path;
	my $ua = LWP::UserAgent->new;
	my @cookiehdr;
	my $index;
	my $content;
	my $statname = "HAPI_STATUS";
	my $value;
	my $ret;
	my $csrftoken;
	my @csrftokenhdr;
	my $checkcert;
	my $certpath;
	
	($cookie, $csrftoken, $url, $checkcert, $certpath) = @_;
	@cookiehdr = ("Cookie" => "SessionCookie=$cookie");
	@csrftokenhdr = ("CSRFTOKEN" => "$csrftoken");
	
	# 証明書の設定を行います
	if ( $url=~m|^https://| ){
		if ( $checkcert ){
			if ( $certpath ne '' ){
				$ua->ssl_opts(SSL_ca_file => $certpath);
			}
		} else {
			$ua->ssl_opts(verify_hostname => 0, SSL_verify_mode => 'SSL_VERIFY_NONE');
		}
	}
	
	# ログアウト処理を行ないます
	{
		my $req;
		my $res;
		my $strname = "WEBVAR_STRUCTNAME_WEB_SESSION_LOGOUT";
		my $result;

		# ログイン処理で得たクッキーをヘッダに付加し、GET要求を発行します
		$path = "$url/rpc/WEBSES/logout.asp";
		$req = GET($path, @cookiehdr, @csrftokenhdr);
		$res = $ua->request($req);
		if (!$res->is_success)
		{
			print $res->status_line,"\n";
			return -1;
		}
		else
		{
			$content = $res->content;

			# BMCから入手した応答を整形します
			($ret, $content) = preparse($content, $strname, $statname);
			if (0 != $ret)
			{
				return $ret;
			}

			# 応答をJSON形式として解析します
			$result = JSON->new->decode($content);
			if ($result->{Error})
			{
				print $result->{Error}{Message}, "\n";
				return -1;
			}
			else
			{
				if (defined($result->{$statname}))
				{
					$value = $result->{$statname};
					if (0 != $value)
					{
						print "$statname error, code $value.\n";
						return -1;
					}
				}
				else
				{
					print "'$statname' is not found.\n";
					print "Invalid response.\n";
					return -1;
				}
			}
		}
	}

	# ログアウト処理は完了しました
	return 0;
}

# JSONデータの整形
sub preparse
{
	my $content;
	my $strname;
	my $statname;
	my $index;

	($content, $strname, $statname) = @_;

	$index = index($content, " { ");
	substr($content, 0, $index-1) = "";

	$index = index($content, ",	 {}");
	substr($content, $index, 5) = "";

	$content =~ s/\'/\"/g;
	$content =~ s/};/}/g;

	$index = index($content, $strname);
	substr($content, $index+length($strname), 0) = "\"";
	substr($content, $index, 0) = "\"";

	$index = index($content, $statname);
	substr($content, $index+length($statname)+1, 0) = " ";
	substr($content, $index+length($statname), 0) = "\" ";
	substr($content, $index, 0) = "\"";
	
	return (0, $content);
}

# CSRFトークンの入手処理
sub getcsrftoken
{
	my $cookie;
	my $url;
	my @cookiehdr;
	my $csrftoken = "";
	my $path;
	my $ua = LWP::UserAgent->new;
	my $content;
	my $statname = "HAPI_STATUS";
	my $array;
	my $value;
	my $ret;
	my $checkcert;
	my $certpath;
	
	($cookie, $url, $checkcert, $certpath) = @_;
	@cookiehdr = ("Cookie" => "SessionCookie=$cookie");
	
	# 証明書の設定を行います
	if ( $url=~m|^https://| ){
		if ( $checkcert ){
			if ( $certpath ne '' ){
				$ua->ssl_opts(SSL_ca_file => $certpath);
			}
		} else {
			$ua->ssl_opts(verify_hostname => 0, SSL_verify_mode => 'SSL_VERIFY_NONE');
		}
	}
	
	# CSRFトークン入手用ASPの実行
	{
		my $req;
		my $res;
		my $strname = "WEBVAR_STRUCTNAME_GETCSRFTOKEN";
		my $resname = "CSRFTOKEN";
		my $result;

		# 必要なパラメータを付加し、POST要求を発行します
		$path = "$url/rpc/getcsrftoken.asp";
		$req = POST($path, @cookiehdr);
		$content = "SESSION_COOKIE=$cookie";
		$req->content_length(length($content));
		$req->content($content);
		$res = $ua->request($req);
		if (!$res->is_success)
		{
			if ( $res->code == 404 ){
				# CSRFトークン非対応
				return (-2, $csrftoken);
			}
			print $path, " : ", $res->status_line, "\n";
			return (-1, $csrftoken);
		}
		else
		{
			$content = $res->content;

			# BMCから入手した応答を整形します
			($ret, $content) = &preparse($content, $strname, $statname);
			if (0 != $ret)
			{
				return ($ret, $csrftoken);
			}

			# 応答をJSON形式として解析します
			$result = JSON->new->decode($content);
			if ($result->{Error})
			{
				print $path, " : ", $result->{Error}{Message}, "\n";
				return (-1, $csrftoken);
			}
			else
			{
				if (defined($result->{$statname}))
				{
					$value = $result->{$statname};
					if (0 != $value)
					{
						print "$path : $statname error, code $value.\n";
						return (-1, $csrftoken);
					}
				}
				else
				{
					print "$path : '$statname' is not found.\n";
					return (-1, $csrftoken);
				}
				if (defined($result->{$strname}))
				{
					$array = $result->{$strname};
					if (defined($array->[0]->{$resname}))
					{
						$value = $array->[0]->{$resname};
					}
					else
					{
						print "$path : '$resname' is not found.\n";
						return (-1, $csrftoken);
					}
				}
				else
				{
					print "$path : '$strname' is not found.\n";
					return (-1, $csrftoken);
				}
				# 入手したCSRFトークンを呼び元へ渡します
				$csrftoken = $value;
			}
		}
	}

	return (0, $csrftoken);
}

# エラーメッセージ出力
sub OutputHttpErrorStatusMessage
{
	my $httpstatus;
	my $ret=-2;
	
	($httpstatus) = @_;
	
	if ( $httpstatus == 400 )
	{
		print "XML file is invalid.\n";
		$ret=-2;
	}
	elsif ( $httpstatus == 403 )
	{
		print "Forbidden. Administrator privilege is required.\n";
		$ret=-2;
	}
	elsif ( $httpstatus == 405 )
	{
		print "Method Not Allowed.\n";
		$ret=-2;
	}
	elsif ( $httpstatus == 409 )
	{
		print "Upload conflict. Please retry a few minutes later.\n";
		$ret=-3;
	}
	elsif ( $httpstatus == 500 )
	{
		print "Internal Server Error. Please retry a few minutes later.\n";
		$ret=-3;
	}
	elsif ( $httpstatus == 503 )
	{
		print "Processing other request. Please retry a few minutes later.\n";
		$ret=-3;
	}
	elsif ( $httpstatus == 590 )
	{
		print "BIOS POST is running. Please retry a few minutes later.\n";
		$ret=-3;
	}
	elsif ( $httpstatus == 591 )
	{
		print "Processing XML. Please retry a few minutes later.\n";
		$ret=-3;
	}
	else
	{
		if ( ($httpstatus>=300)&&($httpstatus<500) ){
			$ret=-2;
		}
		if ( ($httpstatus>=500)&&($httpstatus<600) ){
			$ret=-3;
		}
	}
	
	return $ret;
}

1;


